 /*=================================================================
 *
 * MARKOV.C
 * The calling syntax is:
 *
 *		[iStates] = markov(i0,PI,eps)
 *
 *
 * Input:  i0       1*N, index to initial state
 *         PI       NS*NS, Markov transition matrix
 *         eps      T*N, values on unit interval
 * Output: iStates  T*N, index to states generated by eps
 * =================================================================*/

#include "mex.h"
#include <malloc.h>


/* Output Arguments */
#define	I_OUT	plhs[0]

void mexFunction( int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray *prhs[] )
{          
    double   *I0,*PI,*EPS; 
    long int T,N,NS;
    long int i,j,k;
    double  *STATES;
    long int state;
    double   *CumPI;
    
    
    /* Check for proper number of arguments */    
    if (nrhs != 3) { 
	mexErrMsgTxt("Three input arguments required."); 
    } else if (nlhs != 1) {
	mexErrMsgTxt("One output arguments required."); 
    } 
   
    NS    = mxGetM(prhs[1]);
    T     = mxGetM(prhs[2]);
    N     = mxGetN(prhs[2]);
    
    /* Create a vector for the return argument */ 
    I_OUT = mxCreateDoubleMatrix(T, N, mxREAL); 
    
    /* Assign pointers to the various parameters */ 

    I0    = mxGetPr(prhs[0]);          /* # periods in cycle */
    PI    = mxGetPr(prhs[1]);          /* grid for W*E, i.e. income */
    EPS   = mxGetPr(prhs[2]);          /* # of nodes in WE */
    
    STATES = mxGetPr(I_OUT);
    
        
    CumPI = (double*) calloc(NS*NS,sizeof(double));

    for (i = 0; i < NS; i++)
        for (j = 0; j < NS; j++)
            if (j == 0)
                CumPI[i] = PI[i];
            else
                CumPI[i+j*NS] = CumPI[i+(j-1)*NS] + PI[i+j*NS];
        
    for (k = 0; k < N; k++) {
        state = I0[k] - 1;
        for (i = 0; i < T; i++) {
            for (j = 0; j < NS; j++)
                if (EPS[i+k*T] < CumPI[state+j*NS])
                    break;
            state     = j;
            STATES[i+k*T] = j+1;
        }
    }
    
    
    return;
    
}


